package at.ac.uibk.gitsearch.domain;

import at.ac.uibk.gitsearch.domain.enumeration.ReviewStatus;
import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import java.io.Serializable;
import java.time.Instant;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.ManyToOne;
import javax.persistence.Table;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;
import org.hibernate.annotations.Cache;
import org.hibernate.annotations.CacheConcurrencyStrategy;

/**
 * A ReviewHistoryEntry.
 */
@Entity
@Table(name = "review_history_entry")
@Cache(usage = CacheConcurrencyStrategy.READ_WRITE)
@SuppressWarnings({ "PMD.AvoidFieldNameMatchingMethodName", "PMD.ShortMethodName", "PMD.TooManyMethods" })
public class ReviewHistoryEntry implements Serializable {

    private static final long serialVersionUID = 1L;

    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    @Column(name = "id")
    private Long id;

    @NotNull
    @Size(max = 255)
    @Column(name = "message", length = 255, nullable = false)
    private String message;

    @NotNull
    @Enumerated(EnumType.STRING)
    @Column(name = "type", nullable = false)
    private ReviewStatus type;

    @NotNull
    @Column(name = "entry_date", nullable = false)
    private Instant entryDate = Instant.now();

    @ManyToOne(optional = false)
    @NotNull
    @JsonIgnoreProperties(value = { "comments", "histories" }, allowSetters = true)
    private Review review;

    @ManyToOne
    @JsonIgnoreProperties(value = { "review", "user", "histories" }, allowSetters = true)
    private ReviewRating reviewRating;

    @ManyToOne
    private User user;

    // jhipster-needle-entity-add-field - JHipster will add fields here

    public Long getId() {
        return this.id;
    }

    public ReviewHistoryEntry id(Long id) {
        this.setId(id);
        return this;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public String getMessage() {
        return this.message;
    }

    public ReviewHistoryEntry message(String message) {
        this.setMessage(message);
        return this;
    }

    public void setMessage(String message) {
        this.message = message;
    }

    public ReviewStatus getType() {
        return this.type;
    }

    public ReviewHistoryEntry type(ReviewStatus type) {
        this.setType(type);
        return this;
    }

    public void setType(ReviewStatus type) {
        this.type = type;
    }

    public Instant getEntryDate() {
        return this.entryDate;
    }

    public ReviewHistoryEntry entryDate(Instant entryDate) {
        this.setEntryDate(entryDate);
        return this;
    }

    public void setEntryDate(Instant entryDate) {
        this.entryDate = entryDate;
    }

    public Review getReview() {
        return this.review;
    }

    public void setReview(Review review) {
        this.review = review;
    }

    public ReviewHistoryEntry review(Review review) {
        this.setReview(review);
        return this;
    }

    public ReviewRating getReviewRating() {
        return this.reviewRating;
    }

    public void setReviewRating(ReviewRating reviewRating) {
        this.reviewRating = reviewRating;
    }

    public ReviewHistoryEntry reviewRating(ReviewRating reviewRating) {
        this.setReviewRating(reviewRating);
        return this;
    }

    public User getUser() {
        return this.user;
    }

    public void setUser(User user) {
        this.user = user;
    }

    public ReviewHistoryEntry user(User user) {
        this.setUser(user);
        return this;
    }

    // jhipster-needle-entity-add-getters-setters - JHipster will add getters and setters here

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (!(o instanceof ReviewHistoryEntry)) {
            return false;
        }
        return id != null && id.equals(((ReviewHistoryEntry) o).id);
    }

    @Override
    public int hashCode() {
        // see https://vladmihalcea.com/how-to-implement-equals-and-hashcode-using-the-jpa-entity-identifier/
        return getClass().hashCode();
    }

    // prettier-ignore
    @Override
    public String toString() {
        return "ReviewHistoryEntry{" +
            "id=" + getId() +
            ", message='" + getMessage() + "'" +
            ", type='" + getType() + "'" +
                ", entryDate='" + getEntryDate() + "'" +
            "}";
    }
}
