package at.ac.uibk.gitsearch.service.criteria;

import at.ac.uibk.gitsearch.domain.enumeration.ReviewStatus;
import java.io.Serializable;
import java.util.Objects;
import tech.jhipster.service.Criteria;
import tech.jhipster.service.filter.Filter;
import tech.jhipster.service.filter.InstantFilter;
import tech.jhipster.service.filter.LongFilter;
import tech.jhipster.service.filter.StringFilter;

/**
 * Criteria class for the {@link at.ac.uibk.gitsearch.domain.ReviewHistoryEntry} entity. This class is used
 * in {@link at.ac.uibk.gitsearch.web.rest.ReviewHistoryEntryResource} to receive all the possible filtering options from
 * the Http GET request parameters.
 * For example the following could be a valid request:
 * {@code /review-history-entries?id.greaterThan=5&attr1.contains=something&attr2.specified=false}
 * As Spring is unable to properly convert the types, unless specific {@link Filter} class are used, we need to use
 * fix type specific filters.
 */
@SuppressWarnings(
    {
        "PMD.FieldDeclarationsShouldBeAtStartOfClass",
        "PMD.GodClass",
        "PMD.AvoidFieldNameMatchingMethodName",
        "PMD.ShortMethodName",
        "PMD.TooManyMethods",
        "PMD.NPathComplexity",
        "PMD.ConfusingTernary",
    }
)
public class ReviewHistoryEntryCriteria implements Serializable, Criteria {

    /**
     * Class for filtering ReviewStatus
     */
    public static class ReviewStatusFilter extends Filter<ReviewStatus> {

        private static final long serialVersionUID = 1L;

        public ReviewStatusFilter() {
            super();
        }

        public ReviewStatusFilter(ReviewStatusFilter filter) {
            super(filter);
        }

        @Override
        public ReviewStatusFilter copy() {
            return new ReviewStatusFilter(this);
        }
    }

    private static final long serialVersionUID = 1L;

    private LongFilter id;

    private StringFilter message;

    private ReviewStatusFilter type;

    private InstantFilter entryDate;

    private LongFilter reviewId;

    private LongFilter reviewRatingId;

    private LongFilter userId;

    private String[] sort;

    private Boolean distinct;

    public ReviewHistoryEntryCriteria() {}

    public ReviewHistoryEntryCriteria(ReviewHistoryEntryCriteria other) {
        this.id = other.id == null ? null : other.id.copy();
        this.message = other.message == null ? null : other.message.copy();
        this.type = other.type == null ? null : other.type.copy();
        this.entryDate = other.entryDate == null ? null : other.entryDate.copy();
        this.reviewId = other.reviewId == null ? null : other.reviewId.copy();
        this.reviewRatingId = other.reviewRatingId == null ? null : other.reviewRatingId.copy();
        this.userId = other.userId == null ? null : other.userId.copy();
        this.distinct = other.distinct;
    }

    @Override
    public ReviewHistoryEntryCriteria copy() {
        return new ReviewHistoryEntryCriteria(this);
    }

    public LongFilter getId() {
        return id;
    }

    public LongFilter id() {
        if (id == null) {
            id = new LongFilter();
        }
        return id;
    }

    public void setId(LongFilter id) {
        this.id = id;
    }

    public StringFilter getMessage() {
        return message;
    }

    public StringFilter message() {
        if (message == null) {
            message = new StringFilter();
        }
        return message;
    }

    public void setMessage(StringFilter message) {
        this.message = message;
    }

    public ReviewStatusFilter getType() {
        return type;
    }

    public ReviewStatusFilter type() {
        if (type == null) {
            type = new ReviewStatusFilter();
        }
        return type;
    }

    public void setType(ReviewStatusFilter type) {
        this.type = type;
    }

    public InstantFilter getEntryDate() {
        return entryDate;
    }

    public InstantFilter entryDate() {
        if (entryDate == null) {
            entryDate = new InstantFilter();
        }
        return entryDate;
    }

    public void setEntryDate(InstantFilter entryDate) {
        this.entryDate = entryDate;
    }

    public LongFilter getReviewId() {
        return reviewId;
    }

    public LongFilter reviewId() {
        if (reviewId == null) {
            reviewId = new LongFilter();
        }
        return reviewId;
    }

    public void setReviewId(LongFilter reviewId) {
        this.reviewId = reviewId;
    }

    public LongFilter getReviewRatingId() {
        return reviewRatingId;
    }

    public LongFilter reviewRatingId() {
        if (reviewRatingId == null) {
            reviewRatingId = new LongFilter();
        }
        return reviewRatingId;
    }

    public void setReviewRatingId(LongFilter reviewRatingId) {
        this.reviewRatingId = reviewRatingId;
    }

    public LongFilter getUserId() {
        return userId;
    }

    public LongFilter userId() {
        if (userId == null) {
            userId = new LongFilter();
        }
        return userId;
    }

    public void setUserId(LongFilter userId) {
        this.userId = userId;
    }

    public Boolean getDistinct() {
        return distinct;
    }

    public void setDistinct(Boolean distinct) {
        this.distinct = distinct;
    }

    @SuppressWarnings("PMD.MethodReturnsInternalArray")
    public String[] getSort() {
        return sort;
    }

    @SuppressWarnings("PMD.ArrayIsStoredDirectly")
    public void setSort(String[] sort) {
        this.sort = sort;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (o == null || getClass() != o.getClass()) {
            return false;
        }
        final ReviewHistoryEntryCriteria that = (ReviewHistoryEntryCriteria) o;
        return (
            Objects.equals(id, that.id) &&
            Objects.equals(message, that.message) &&
            Objects.equals(type, that.type) &&
            Objects.equals(entryDate, that.entryDate) &&
            Objects.equals(reviewId, that.reviewId) &&
            Objects.equals(reviewRatingId, that.reviewRatingId) &&
            Objects.equals(userId, that.userId) &&
            Objects.equals(sort, that.sort) &&
            Objects.equals(distinct, that.distinct)
        );
    }

    @Override
    public int hashCode() {
        return Objects.hash(id, message, type, entryDate, reviewId, reviewRatingId, userId, sort, distinct);
    }

    // prettier-ignore
    @Override
    @SuppressWarnings("PMD.CyclomaticComplexity")
    public String toString() {
        return "ReviewHistoryEntryCriteria{" +
            (id != null ? "id=" + id + ", " : "") +
            (message != null ? "message=" + message + ", " : "") +
            (type != null ? "type=" + type + ", " : "") +
            (entryDate != null ? "entryDate=" + entryDate + ", " : "") +
            (reviewId != null ? "reviewId=" + reviewId + ", " : "") +
            (reviewRatingId != null ? "reviewRatingId=" + reviewRatingId + ", " : "") +
            (userId != null ? "userId=" + userId + ", " : "") +
                (sort != null ? "sort=" + sort + ", " : "") +
            (distinct != null ? "distinct=" + distinct + ", " : "") +
            "}";
    }
}
