package at.ac.uibk.gitsearch.web.rest;

import at.ac.uibk.gitsearch.properties.ApplicationProperties;
import at.ac.uibk.gitsearch.properties.ApplicationProperties.DeploymentInfo;
import at.ac.uibk.gitsearch.security.AuthoritiesConstants;
import at.ac.uibk.gitsearch.service.MessageService;
import at.ac.uibk.gitsearch.service.dto.BroadCastMessageDTO;
import at.ac.uibk.gitsearch.service.util.UtilityService;
import java.util.List;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * REST controller to retrieve deployment infos.
 */
@RestController
@RequestMapping("/api")
public class ApplicationManagementResource {

    @SuppressWarnings("unused")
    private final Logger log = LoggerFactory.getLogger(ApplicationManagementResource.class);

    private final UtilityService utilityService;

    private final ApplicationProperties applicationProperties;
    private final MessageService messageService;

    public ApplicationManagementResource(
        ApplicationProperties applicationProperties,
        MessageService messageService,
        UtilityService utilityService
    ) {
        this.applicationProperties = applicationProperties;
        this.messageService = messageService;
        this.utilityService = utilityService;
    }

    /**
     * {@code GET  /applicationInfo/deploymentInfo} : get generic deployment info.
     *
     */
    @GetMapping("/applicationInfo/deploymentInfo")
    public ApplicationProperties.DeploymentInfo getApplicationInfo() {
        final DeploymentInfo deploymentInfo = applicationProperties.getDeploymentInfo();
        if (deploymentInfo == null || "${gitBranch}".equals(deploymentInfo.getBranch())) {
            return new ApplicationProperties.DeploymentInfo(); // application info is not initialized properly! Return an empty info
        }
        return deploymentInfo;
    }

    @GetMapping("/applicationInfo/oerLinkInfo")
    public String getOerLink() {
        return applicationProperties.getOeResource().getOerLink();
    }

    /**
     * {@code GET  /applicationInfo/deploymentInfo} : get generic deployment info.
     *
     */
    @GetMapping("/applicationInfo/broadcastMessages")
    public BroadCastMessageDTO[] getBroadCastMessages() {
        return messageService.getMessages();
    }

    /**
     * {@code DELETE  /applicationInfo/syncEntities} : synchronizes Entities
     * from DB with elastic Search.
     *
     */
    @DeleteMapping("/admin/utility/syncEntities")
    @PreAuthorize("hasAuthority(\"" + AuthoritiesConstants.ADMIN + "\")")
    public List<String> syncEntities() {
        return utilityService.synchronizeElasticSearchFromEntities();
    }
}
