import { Component, OnChanges, OnInit } from '@angular/core';
import { CatAliasesAliasesRecord, CatIndicesIndicesRecord } from '@elastic/elasticsearch/lib/api/types';
import { interval, startWith, Subscription } from 'rxjs';
import { IndexManagementService, LoggerInfo } from './index-management.service';

@Component({
  selector: 'jhi-index-management',
  templateUrl: './index-management.component.html',
  styleUrls: ['./index-management.component.scss'],
})
export class IndexManagementComponent implements OnInit, OnChanges {
  indices: CatIndicesIndicesRecord[] = [];
  aliases: CatAliasesAliasesRecord[] = [];

  reIndexingRunning = false;
  logPolling: Subscription | undefined;
  logId: string | undefined;
  indexLog = '';

  constructor(private indexManagementService: IndexManagementService) {}

  ngOnInit(): void {
    this.loadAll();
  }

  ngOnChanges() {
    this.loadAll();
  }

  delete(index: string): void {
    this.indexManagementService.deleteIndex(index).subscribe({
      next: result => {
        if (!result) {
          alert('Deletion of index ' + index + ' failed.');
        }
        this.loadAll();
      },
      error: err => {
        alert('Deletion of index ' + index + ' failed: ' + String(err));
      },
    });
  }

  isIndexDeletable(index: string): boolean {
    if (index.startsWith('idx_metadata') && !this.isActiveIndex(index)) {
      return true;
    }
    return false;
  }

  isActiveIndex(index: string) {
    for (const alias of this.aliases) {
      if (alias.alias == 'metadata' && alias.index == index) {
        return true;
      }
    }
    return false;
  }

  loadAll(): void {
    this.indexManagementService.getIndices().subscribe((data: CatIndicesIndicesRecord[]) => {
      this.indices = data;
    });
    this.indexManagementService.getAliasses().subscribe((data: CatAliasesAliasesRecord[]) => {
      this.aliases = data;
    });
  }

  reIndex(): void {
    this.indexManagementService.reindex().subscribe((data: string) => {
      this.reIndexingRunning = true;
      this.logId = data;
      this.indexLog = '';
      this.logPolling = interval(5000)
        .pipe(startWith(0))
        .subscribe({
          next: () => {
            this.indexManagementService.getLog(this.logId!).subscribe((log: LoggerInfo) => {
              this.indexLog += log.log;
              if (log.closed) {
                this.logPolling?.unsubscribe();
                this.loadAll();
                this.reIndexingRunning = false;
              }
            });
          },
          error: err => {
            this.logPolling?.unsubscribe();
            this.loadAll();
            this.reIndexingRunning = false;
            this.indexLog += 'Error: ' + String(err.message);
          },
        });
    });
  }
}
