import { Component, OnInit } from '@angular/core';
import { HttpResponse } from '@angular/common/http';
import { NgbModal } from '@ng-bootstrap/ng-bootstrap';

import { IReviewHistoryEntry } from '../review-history-entry.model';
import { ReviewHistoryEntryService } from '../service/review-history-entry.service';
import { ReviewHistoryEntryDeleteDialogComponent } from '../delete/review-history-entry-delete-dialog.component';

@Component({
  selector: 'jhi-review-history-entry',
  templateUrl: './review-history-entry.component.html',
})
export class ReviewHistoryEntryComponent implements OnInit {
  reviewHistoryEntries?: IReviewHistoryEntry[];
  isLoading = false;

  constructor(protected reviewHistoryEntryService: ReviewHistoryEntryService, protected modalService: NgbModal) {}

  loadAll(): void {
    this.isLoading = true;

    this.reviewHistoryEntryService.query().subscribe({
      next: (res: HttpResponse<IReviewHistoryEntry[]>) => {
        this.isLoading = false;
        this.reviewHistoryEntries = res.body ?? [];
      },
      error: () => {
        this.isLoading = false;
      },
    });
  }

  ngOnInit(): void {
    this.loadAll();
  }

  trackId(index: number, item: IReviewHistoryEntry): number {
    return item.id!;
  }

  delete(reviewHistoryEntry: IReviewHistoryEntry): void {
    const modalRef = this.modalService.open(ReviewHistoryEntryDeleteDialogComponent, { size: 'lg', backdrop: 'static' });
    modalRef.componentInstance.reviewHistoryEntry = reviewHistoryEntry;
    // unsubscribe not needed because closed completes on modal close
    modalRef.closed.subscribe(reason => {
      if (reason === 'deleted') {
        this.loadAll();
      }
    });
  }
}
