import { Injectable } from '@angular/core';
import { HttpClient, HttpResponse } from '@angular/common/http';
import { Observable } from 'rxjs';

import { isPresent } from 'app/core/util/operators';
import { ApplicationConfigService } from 'app/core/config/application-config.service';
import { createRequestOption } from 'app/core/request/request-util';
import { IReviewRating, getReviewRatingIdentifier } from '../review-rating.model';

export type EntityResponseType = HttpResponse<IReviewRating>;
export type EntityArrayResponseType = HttpResponse<IReviewRating[]>;

@Injectable({ providedIn: 'root' })
export class ReviewRatingService {
  protected resourceUrl = this.applicationConfigService.getEndpointFor('api/review-ratings');

  constructor(protected http: HttpClient, protected applicationConfigService: ApplicationConfigService) {}

  create(reviewRating: IReviewRating): Observable<EntityResponseType> {
    return this.http.post<IReviewRating>(this.resourceUrl, reviewRating, { observe: 'response' });
  }

  update(reviewRating: IReviewRating): Observable<EntityResponseType> {
    return this.http.put<IReviewRating>(`${this.resourceUrl}/${getReviewRatingIdentifier(reviewRating) as number}`, reviewRating, {
      observe: 'response',
    });
  }

  partialUpdate(reviewRating: IReviewRating): Observable<EntityResponseType> {
    return this.http.patch<IReviewRating>(`${this.resourceUrl}/${getReviewRatingIdentifier(reviewRating) as number}`, reviewRating, {
      observe: 'response',
    });
  }

  find(id: number): Observable<EntityResponseType> {
    return this.http.get<IReviewRating>(`${this.resourceUrl}/${id}`, { observe: 'response' });
  }

  query(req?: any): Observable<EntityArrayResponseType> {
    const options = createRequestOption(req);
    return this.http.get<IReviewRating[]>(this.resourceUrl, { params: options, observe: 'response' });
  }

  delete(id: number): Observable<HttpResponse<{}>> {
    return this.http.delete(`${this.resourceUrl}/${id}`, { observe: 'response' });
  }

  addReviewRatingToCollectionIfMissing(
    reviewRatingCollection: IReviewRating[],
    ...reviewRatingsToCheck: (IReviewRating | null | undefined)[]
  ): IReviewRating[] {
    const reviewRatings: IReviewRating[] = reviewRatingsToCheck.filter(isPresent);
    if (reviewRatings.length > 0) {
      const reviewRatingCollectionIdentifiers = reviewRatingCollection.map(
        reviewRatingItem => getReviewRatingIdentifier(reviewRatingItem)!
      );
      const reviewRatingsToAdd = reviewRatings.filter(reviewRatingItem => {
        const reviewRatingIdentifier = getReviewRatingIdentifier(reviewRatingItem);
        if (reviewRatingIdentifier == null || reviewRatingCollectionIdentifiers.includes(reviewRatingIdentifier)) {
          return false;
        }
        reviewRatingCollectionIdentifiers.push(reviewRatingIdentifier);
        return true;
      });
      return [...reviewRatingsToAdd, ...reviewRatingCollection];
    }
    return reviewRatingCollection;
  }
}
